import pygame, sys, os.path
from math import *
from constants import *
import time

# -- Graphics --
# Creates window and handles drawing events

def timing(f):
    def wrap(*args):
        time1 = time.time()
        ret = f(*args)
        time2 = time.time()
        #print '%s function took %0.3f ms' % (f.func_name, (time2-time1)*1000.0)
        return ret
    return wrap

class Graphics():
    def __init__(self, w, h, fps, title):
        # Window caption
        pygame.display.set_caption(title)

        # Screen size
        self.screen = pygame.display.set_mode((w, h))#, pygame.DOUBLEBUF)
        self.cam_x = 0
        self.cam_y = 0

        # Fps
        self.clock = pygame.time.Clock()
        self.fps = fps
        self.fpslist = []
        self.msElapsed = 0
        self.step = 0

        # Images
        self.images = {}
        path = 'data/images/'
        self.load_texture('platform', path + 'Platform.png', True)
        for i in range(imageAmount['castle']):
            self.load_texture('castle{}'.format(i+1), path + 'Castle_{}.png'.format(i+1), True)
        for i in range(imageAmount['cloud']):
            self.load_texture('cloud{}'.format(i+1), path + 'Cloud_{}.png'.format(i+1), True)
        for i in range(imageAmount['brick']):
            self.load_texture('brick{}'.format(i+1), path + 'Brick_{}.png'.format(i+1), True)
        for i in range(imageAmount['flare']):
            self.load_texture('flare{}'.format(i+1), path + 'Flare_{}.png'.format(i+1), True)
        for i in range(imageAmount['hill']):
            self.load_texture('hill{}'.format(i+1), path + 'Hill_{}.png'.format(i+1), True)
        
        # Font
        self.fonts = {}
        path = 'data/fonts/'
        self.load_font('basic', path + 'visitor.ttf', 30, False)
        self.load_font('zelda', path + 'triforce.ttf', 60, False)
        self.load_font('Times New Roman', path + 'Times_New_Roman.ttf', 30, False)
        

    def createBackground(self, effects):
        w = wbcw
        h = wbch
        self.background = pygame.Surface((w, h))
        t = 50.0
        c1 = (80, 225, 255)
        c2 = (0, 96, 160)
        colors = [[int(c1[i] - (c1[i]-c2[i]) * e/t) for i in range(3)] for e in range(int(t))]
        for i in range(len(colors)):
            y = h * i / float(len(colors))
            rect = [0, y, w, h]
            pygame.draw.rect(self.background, colors[i], rect)
        for e in effects:
            image = self.images['cloud{}'.format(e.image)]
            image = pygame.transform.flip(image, e.flip, False)
            pos = (e.x, e.y)
            self.background.blit(image, pos)
        
    # Refresh display and keep fps
    def update(self):
        self.step += 1
        self.msElapsed = self.clock.tick(self.fps)
        pygame.display.update()

    # Load an image into self.images
    def load_texture(self, name, path, alpha):
        if not os.path.isfile(path):
            print '[Error] Image \'{}\' not found at \'{}\''.format(name, path)
            return
        image = pygame.image.load(path)
        image.convert_alpha()
        self.images[name] = image
        #self.screen.fill(c_white)
        
    # Load a font into self.fonts
    def load_font(self, name, path, size, bold):
        if not os.path.isfile(path):
            print '[Error] Font \'{}\' not found at \'{}\''.format(name, path)
            return
        font = pygame.font.Font(path, size)
        if bold: font.set_bold(True)
        self.fonts[name] = font
    
    # Shift (rotated) object to its center
    def shift_center(self, pos, image):
        shift = (- image.get_width() / 2.0, - image.get_height() / 2.0)
        return map(sum, zip(pos, shift))
        
    # Scale image after scale-factors
    def image_scale(self, image, xscale, yscale):
        return pygame.transform.scale(image, (
            int(xscale * image.get_width()),
            int(yscale * image.get_height())
        ))

    # Flip y-axis
    def flipyv(self, v):
        return v[0], screen_height - v[1]
    
    # Move with cam
    def cam_move(self, v, s=1.0):
        if isinstance(v, list) or isinstance(v, tuple):
            if isinstance(v[0], list) or isinstance(v[0], tuple):
                return map(self.cam_move, v)
            return v[0] - s*self.cam_x, v[1] - s*self.cam_y

    # Check if within screen
    def withinView(self, pos, image):
        if pos[0] > screen_width:
            return False
        if pos[1] > screen_height:
            return False
        if pos[0] + image.get_width() < 0:
            return False
        if pos[1] + image.get_height() < 0:
            return False
        return True

        
    # Draw background example
    @timing
    def draw_background(self):
        self.screen.blit(self.background, self.cam_move((0,0), bgscale))
        
           
    @timing 
    def draw_fps(self):
        self.fpslist.append(1000/self.msElapsed)
        while len(self.fpslist) > 10:
            self.fpslist.pop(0)
        s = str(int(sum(self.fpslist)/len(self.fpslist)))
        text = self.fonts['basic'].render(s, True, c_black)
        # Adjust positions with rect-object: right, left, top, bottom, centerx, centery
        rect = text.get_rect()
        rect.right = screen_width
        rect.top = 0
        self.screen.blit(text, rect)

    @timing
    def draw_physics(self, physics, damageStage):            
        image = self.images['castle{}'.format(damageStage)]
        image = self.image_scale(image, 0.5, 0.5)
        angle = physics.player.angle
        image = pygame.transform.rotate(image, degrees(angle))
        pos = physics.getPlayerCenter()
        self.screen.blit(image, self.cam_move(self.shift_center(pos, image)))
    
        if False:
            for shape in physics.space.shapes:
                if shape.body.is_static:
                    body = shape.body
                    pv1 = physics.flipyv(body.position + shape.a.cpvrotate(body.rotation_vector))
                    pv2 = physics.flipyv(body.position + shape.b.cpvrotate(body.rotation_vector))
                    pygame.draw.lines(self.screen, c_red, False, [self.cam_move((pv1[0]-1,pv1[1]-1)),self.cam_move((pv2[0]-1,pv2[1]-1))], 2)
                else:
                    if shape.body.is_sleeping:
                        continue
                    ps = shape.get_vertices()
                    ps = map(physics.flipyv, ps)
                    pygame.draw.polygon(self.screen, c_red, self.cam_move(ps), 2)
            for c in physics.collisions:
                pygame.draw.circle(self.screen, c_red, map(int, self.cam_move(c[0])), c[1], 0)
           
    @timing 
    def draw_bricks(self, effects):
        for e in effects:
            image = self.images['brick{}'.format(e.image)]
            image = pygame.transform.flip(image, e.flip, False)
            image = self.image_scale(image, e.r, e.r)
            image = pygame.transform.rotate(image, degrees(e.angle))
            pos = (e.x, e.y)
            self.screen.blit(image, self.cam_move(self.shift_center(pos, image)))
    
    @timing
    def draw_flares(self, effects):
        for e in effects:
            image = self.images['flare{}'.format(e.image)]
            image = self.image_scale(image, e.r, e.r)
            image = pygame.transform.rotate(image, degrees(e.angle))
            pos = (e.x, e.y)
            if e.flip:
                arr = pygame.surfarray.pixels3d(image)
                for j in range(3):
                    arr[:,:,j] *= (255,128,0)[j]/255.0
                del arr
            pos = self.cam_move(self.shift_center(pos, image))
            self.screen.blit(image, pos)
        
    @timing
    def draw_clouds(self, effects):
        for e in effects:
            image = self.images['cloud{}'.format(e.image)]
            image = pygame.transform.flip(image, e.flip, False)
            pos = (e.x, e.y)
            if self.withinView(pos, image):
                self.screen.blit(image, pos)
    
    @timing
    def draw_platforms(self, platforms, claimed):
        for p in platforms:
            image = self.images['platform']
            pos = self.cam_move(self.flipyv(p.position))
            image = self.image_scale(image, 0.75, 0.75)
            if p in claimed:
                arr = pygame.surfarray.pixels3d(image)
                for j in range(3):
                    arr[:,:,j] *= (255,255,0)[j]/255.0
                del arr
            if self.withinView(pos, image):
                self.screen.blit(image, pos)

    @timing
    def draw_hills(self, hills):
        for h in hills:
            image = self.images['hill{}'.format(h[1])]
            pos = self.cam_move(self.flipyv(h[0].position))
            if self.withinView(pos, image):
                self.screen.blit(image, pos)

    def cam_follow_player(self, player):
        x = player.position[0]
        y = player.position[1]
        t = 300
        k = 8.0
        if self.cam_x > x - t:
            self.cam_x -= abs(self.cam_x - (x - t)) / k
        if self.cam_y > screen_height - y - t:
            self.cam_y -= abs(self.cam_y - (screen_height - y - t)) / k
        if self.cam_x < x - screen_width + t:
            self.cam_x += abs(self.cam_x - (x - screen_width + t)) / k
        if self.cam_y < - y + t:
            self.cam_y += abs(self.cam_y - (- y + t)) / k
        if self.cam_x < wbcx:
            self.cam_x = wbcx
        if self.cam_y < wbcy:
            self.cam_y = wbcy
        if self.cam_x > wbcw - screen_width:
            self.cam_x = wbcw - screen_width
        if self.cam_y > wbch - screen_height:
            self.cam_y = wbch - screen_height
